@@ -39,20 +39,20 @@
  *
  * ------------------------------------
  *
- * 
+ *
  * ========LICENCE========
  * This file is part of the library LinBox.
- * 
+ *
  * LinBox is free software: you can redistribute it and/or modify
  * it under the terms of the  GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -71,14 +71,14 @@
 #ifndef __LINBOX_vector_stream_H
 #define __LINBOX_vector_stream_H
 
+
+#include <givaro/givranditer.h>
+#include "linbox/util/debug.h"
+#include "linbox/randiter/mersenne-twister.h"
+#include "linbox/vector/vector-traits.h"
 #include <vector>
 #include <cmath>
 
-#include "linbox/util/debug.h"
-#include "linbox/vector/vector-traits.h"
-#include "linbox/randiter/nonzero.h"
-#include "linbox/randiter/mersenne-twister.h"
-
 
 // stream
 namespace LinBox
@@ -172,7 +172,15 @@ namespace LinBox
 		 * @param v Vector to use
 		 */
 		Vector &get (Vector &v)
-		{ if (_m == 0 || _j < _m) copy (_v.begin (), _v.end (), v.begin ()); return v; }
+		{
+			if (_m == 0 || _j < _m)
+				std::copy (_v.begin (), _v.end (), v.begin ());
+#ifdef _LB_DEBUG
+			else {
+				std::cerr << "Vector stream: nothing to get" << std::endl;
+			}
+#endif
+			return v; }
 
 		/** Extraction operator form
 		*/
@@ -205,26 +213,24 @@ namespace LinBox
 	};
 }
 
+
+//#include "linbox/vector/blas-vector.h"
 // Dense
 namespace LinBox
 {
+	template<class _Field, class _Rep> class BlasVector ;
+
 	/** @brief Random dense vector stream.
 	 * Generates a sequence of random dense vectors over a given field
 	 */
-	template <class Field, class _Vector = typename LinBox::Vector<Field>::Dense, class RandIter = typename Field::RandIter, class Trait = typename VectorTraits<_Vector>::VectorCategory>
+	template <typename Field, typename _Vector = BlasVector<Field, typename Vector<Field>::Dense>,
+              class RandIter = typename Field::RandIter,
+              class Trait = typename VectorTraits<_Vector>::VectorCategory>
 	class RandomDenseStream : public VectorStream<_Vector> {
 	public:
 		typedef _Vector Vector;
 		typedef RandomDenseStream<Field, Vector, RandIter, Trait> Self_t;
 
-		/** Constructor
-		 * Construct a new stream with the given field and vector size.
-		 * @param F Field over which to create random vectors
-		 * @param n Size of vectors
-		 * @param m Number of vectors to return (0 for unlimited)
-		 */
-		RandomDenseStream (const Field &F, size_t n, size_t m = 0);
-
 		/** Constructor.
 		 * Construct a new stream with the given field and vector size.
 		 * @param F Field over which to create random vectors
@@ -232,7 +238,7 @@ namespace LinBox
 		 * @param n Size of vectors
 		 * @param m Number of vectors to return (0 for unlimited)
 		 */
-		RandomDenseStream (const Field &F, const RandIter &r, size_t n, size_t m = 0);
+		RandomDenseStream (const Field &F, RandIter &r, size_t n, size_t m = 0);
 
 		/** Get next element
 		 * @param v Vector into which to generate random vector
@@ -272,11 +278,7 @@ namespace LinBox
 		typedef _Vector Vector;
 		typedef RandomDenseStream<Field, Vector, RandIter, VectorCategories::DenseVectorTag > Self_t;
 
-		RandomDenseStream (const Field &F, size_t nn, size_t mm = 0) :
-			_field (F), _r (F), _n (nn), _m (mm), _j (0)
-		{}
-
-		RandomDenseStream (const Field &F, const RandIter &r, size_t nn, size_t mm = 0) :
+		RandomDenseStream (const Field &F, RandIter &r, size_t nn, size_t mm = 0) :
 			_field (F), _r (r), _n (nn), _m (mm), _j (0)
 		{}
 
@@ -287,7 +289,7 @@ namespace LinBox
 			if ( (_m > 0) && (_j++ >= _m) )
 				return v;
 
-			for (i = v.begin (); i != v.end (); i++)
+			for (i = v.begin (); i != v.end (); ++i)
 				_r.random (*i);
 
 			return v;
@@ -305,7 +307,7 @@ namespace LinBox
 
 	private:
 		const Field &_field;
-		RandIter     _r;
+		RandIter     & _r;
 		size_t       _n;
 		size_t       _m;
 		size_t       _j;
@@ -313,27 +315,19 @@ namespace LinBox
 
 }
 
+#include "linbox/vector/sparse.h"
 //Sparse
 namespace LinBox
 {
 	/** @brief Random sparse vector stream.
 	 * Generates a sequence of random sparse vectors over a given field
 	 */
-	template <class Field, class _Vector = typename LinBox::Vector<Field>::Sparse, class RandIter = typename Field::RandIter, class Trait = typename VectorTraits<_Vector>::VectorCategory>
+	template <class Field, class _Vector = Sparse_Vector<typename Field::Element>, class RandIter = typename Field::RandIter, class Trait = typename VectorTraits<_Vector>::VectorCategory>
 	class RandomSparseStream : public VectorStream<_Vector> {
 	public:
 		typedef _Vector Vector;
 		typedef RandomSparseStream<Field, Vector, RandIter, Trait > Self_t;
 
-		/** Constructor
-		 * Construct a new stream with the given field and vector size.
-		 * @param F Field over which to create random vectors
-		 * @param p Proportion of nonzero entries
-		 * @param n Size of vectors
-		 * @param m Number of vectors to return (0 for unlimited)
-		 */
-		RandomSparseStream (const Field &F, double p, size_t n, size_t m = 0);
-
 		/** Constructor.
 		 * Construct a new stream with the given field and vector size.
 		 * @param F Field over which to create random vectors
@@ -343,7 +337,7 @@ namespace LinBox
 		 * @param m Number of vectors to return (0 for unlimited)
 		 * @param seed
 		 */
-		RandomSparseStream (const Field &F, const RandIter &r, double p, size_t n, size_t m = 0, int seed=(int)time (NULL));
+		RandomSparseStream (const Field &F, RandIter &r, double p, size_t n, size_t m = 0, uint64_t seed=(uint64_t)time (NULL));
 
 		/** Get next element
 		 * @param v Vector into which to generate random vector
@@ -387,37 +381,27 @@ namespace LinBox
 		typedef _Vector Vector;
 		typedef RandomSparseStream<Field, Vector, RandIter, VectorCategories::DenseVectorTag > Self_t;
 
-		RandomSparseStream (const Field &F, double p, size_t N, size_t M = 0) :
-			_field (F), _r1 (F), _r (F, _r1),
-			_n (N), _p (p), _m (M), _j (0),
-			MT ((uint32_t)time (NULL))
+		RandomSparseStream (const Field &F, RandIter &r, double p, size_t n, size_t m = 0, uint64_t seed=(uint64_t)time (NULL)) :
+			_field (F), _r1 (r), _r (_r1), _n (n), _p (p), _m (m), _j (0),
+			MT (static_cast<uint32_t>(seed))
 		{
 			linbox_check ((p >= 0.0) && (p <= 1.0));
-			_field.init (_zero, 0);
-		}
-
-		RandomSparseStream (const Field &F, const RandIter &r, double p, size_t n, size_t m = 0, int seed=(int)time (NULL)) :
-			_field (F), _r1 (r), _r (F, _r1), _n (n), _p (p), _m (m), _j (0),
-			MT ((uint32_t)seed)
-		{
-			linbox_check ((p >= 0.0) && (p <= 1.0));
-			_field.init (_zero, 0);
 		}
 
 		Vector &get (Vector &v)
 		{
-			double val;
 
 			if (_m > 0 && _j++ >= _m)
 				return v;
 
 			for (typename Vector::iterator i = v.begin (); i != v.end (); ++i) {
+				double val;
 				val = MT.randomDouble ();
 
 				if (val < _p)
 					_r.random (*i);
 				else
-					_field.assign (*i, _zero);
+					_field.assign (*i, _field.zero);
 			}
 
 			return v;
@@ -436,9 +420,8 @@ namespace LinBox
 
 	private:
 		const Field                      &_field;
-		typename Field::Element           _zero;
-		RandIter                          _r1;
-		NonzeroRandIter<Field, RandIter>  _r;
+		RandIter                         & _r1;
+        Givaro::GeneralRingNonZeroRandIter<Field, RandIter>  _r;
 		size_t                            _n;
 		double                            _p;
 		size_t                            _m;
@@ -450,26 +433,25 @@ namespace LinBox
 	template <class Field, class _Vector, class RandIter>
 	class RandomSparseStream<Field, _Vector, RandIter, VectorCategories::SparseSequenceVectorTag > : public VectorStream<_Vector> {
 	public:
+		typedef RandomSparseStream<Field, _Vector, RandIter, VectorCategories::SparseSequenceVectorTag > Self_t;
 		typedef _Vector Vector;
-		typedef RandomSparseStream<Field, Vector, RandIter, VectorCategories::SparseSequenceVectorTag > Self_t;
 
-		RandomSparseStream (const Field &F, double p, size_t N, size_t M = 0) :
-			_field (F), _r1 (F), _r (F, _r1), _n (N), _m (M), _j (0),
-			MT ((uint32_t)time (NULL))
-		{ setP (p); }
-
-		RandomSparseStream (const Field &F, const RandIter &r, double p,
-				    size_t N, size_t M = 0, int seed=(int)time (NULL)) :
-			_field (F), _r1 (r), _r (F, _r1), _n (N), _p (p), _m (M), _j (0),
-			MT ((uint32_t)seed)
+		RandomSparseStream (const Field &F, RandIter &r, double p,
+				    size_t N, size_t M = 0, uint64_t seed= time (NULL)) :
+                _field (F), 
+                _r1 (r), 
+                _r (r), 
+                _n (N), 
+                _p (p), 
+                _m (M), 
+                _j (0),
+                MT (static_cast<uint32_t>(seed))
 		{ setP (p); }
 
 		Vector &get (Vector &v)
 		{
 			typename Field::Element x;
 			size_t i = (size_t) -1;
-			double val;
-			int skip;
 
 			if (_m > 0 && _j++ >= _m)
 				return v;
@@ -477,6 +459,10 @@ namespace LinBox
 			v.clear ();
 
 			while (1) {
+
+				double val;
+				int skip;
+
 				val = MT.randomDouble ();
 				skip = (int) (ceil (log (val) * _1_log_1mp));
 
@@ -513,8 +499,8 @@ namespace LinBox
 
 	private:
 		const Field                      &_field;
-		RandIter                          _r1;
-		NonzeroRandIter<Field, RandIter>  _r;
+		RandIter                         & _r1;
+		Givaro::GeneralRingNonZeroRandIter<Field, RandIter>  _r;
 		size_t                            _n;
 		double                            _p;
 		double                            _1_log_1mp;
@@ -530,29 +516,24 @@ namespace LinBox
 		typedef _Vector Vector;
 		typedef RandomSparseStream<Field, Vector, RandIter, VectorCategories::SparseAssociativeVectorTag > Self_t;
 
-		RandomSparseStream (const Field &F, double p, size_t N, size_t M = 0) :
-			_field (F), _r1 (F), _r (F, _r1),
-			_n (N), _k ((long) (p * (double)N)), _j (0), _m (M),
-			MT ((uint32_t)time (NULL))
-		{}
-
-		RandomSparseStream (const Field &F, const RandIter &r, double p, size_t N,
-				    size_t M = 0, int seed=(int)time (NULL)) :
-			_field (F), _r1 (F), _r (F, _r1), _n (N), _k ((long) (p * N)), _j (0), _m (M),
-			MT ((uint32_t) seed)
+		RandomSparseStream (const Field &F, RandIter &r, double p, size_t N,
+				    size_t M = 0, uint64_t seed= time (NULL)) :
+			_field (F), _r1 (r), _r (_r1), _n (N), _k ((long) (p * N)), _j (0), _m (M),
+			MT (static_cast<uint32_t>(seed))
 		{}
 
 		Vector &get (Vector &v)
 		{
 			typename Field::Element x;
-			int i, idx;
+			int i;
 
 			if (_m > 0 && _j++ >= _m)
 				return v;
 
 			v.clear ();
 
-			for (i = 0; i < _k; i++) {
+			for (i = 0; i < _k; ++i) {
+				size_t idx;
 				_r.random (x);
 				while (!_field.isZero (v[idx = MT.randomIntRange (0, (uint32_t)_n)])) ;
 				v[idx] = x;
@@ -574,8 +555,8 @@ namespace LinBox
 
 	private:
 		const Field                      &_field;
-		RandIter                          _r1;
-		NonzeroRandIter<Field, RandIter>  _r;
+		RandIter                         & _r1;
+		Givaro::GeneralRingNonZeroRandIter<Field, RandIter> _r;
 		size_t                            _n;
 		long                              _k;
 		size_t                            _j;
@@ -590,22 +571,15 @@ namespace LinBox
 		typedef _Vector Vector;
 		typedef RandomSparseStream<Field, Vector, RandIter, VectorCategories::SparseParallelVectorTag > Self_t;
 
-		RandomSparseStream (const Field &F, double p, size_t nn, size_t mm = 0) :
-			_field (F), _r1 (F), _r (F, _r1), _n (nn), _m (mm), _j (0),
-			MT ((uint32_t)time (NULL))
-		{ setP (p); }
-
-		RandomSparseStream (const Field &F, const RandIter &r, double p, size_t nn, size_t mm = 0, int seed=(int)time (NULL)) :
-			_field (F), _r1 (r), _r (F, _r1), _n (nn), _m (mm), _j (0),
-			MT ((uint32_t)seed)
+		RandomSparseStream (const Field &F, RandIter &r, double p, size_t nn, size_t mm = 0, uint64_t  seed= time (NULL)) :
+			_field (F), _r1 (r), _r (_r1), _n (nn), _m (mm), _j (0),
+			MT (static_cast<uint32_t>(seed))
 		{ setP (p); }
 
 		Vector &get (Vector &v)
 		{
 			typename Field::Element x;
 			size_t i = (size_t) -1;
-			double val;
-			int skip;
 
 			if (_m > 0 && _j++ >= _m)
 				return v;
@@ -614,11 +588,15 @@ namespace LinBox
 			v.second.clear ();
 
 			while (1) {
+
+				double val;
+			int skip;
+
 				val = MT.randomDouble ();
 				skip = (int) (ceil (log (val) * _1_log_1mp));
 
 				if (skip <= 0)
-					i++;
+					++i;
 				else
 					i += (size_t) skip;
 
@@ -652,8 +630,8 @@ namespace LinBox
 
 	private:
 		const Field                      &_field;
-		RandIter                          _r1;
-		NonzeroRandIter<Field, RandIter>  _r;
+		RandIter                         & _r1;
+        Givaro::GeneralRingNonZeroRandIter<Field, RandIter>  _r;
 		size_t                            _n;
 		double                            _p;
 		double                            _1_log_1mp;
@@ -734,18 +712,17 @@ namespace LinBox
 
 		Vector &get (Vector &v)
 		{
-			static typename Field::Element zero;
 			typename Vector::iterator i;
-			size_t idx;
 
-			for (i = v.begin (), idx = 0; i != v.end (); i++, idx++) {
+			size_t idx;
+			for (i = v.begin (), idx = 0; i != v.end (); ++i, ++idx) {
 				if (idx == _j)
-					_field.init (*i, 1);
+					_field.assign(*i, _field.one);
 				else
-					_field.assign (*i, zero);
+					_field.assign (*i, _field.zero);
 			}
 
-			_j++;
+			++_j;
 
 			return v;
 		}
@@ -753,11 +730,19 @@ namespace LinBox
 		/** Extraction operator form
 		*/
 		Self_t &operator >> (Vector &v)
-		{ get (v); return *this; }
+		{
+			get (v);
+			return *this;
+		}
+
 		size_t size () const { return _n; }
+
 		size_t pos () const { return _j; }
+
 		size_t dim () const { return _n; }
+
 		operator bool () const { return _j < _n; }
+
 		void reset () { _j = 0; }
 
 	private:
@@ -775,14 +760,15 @@ namespace LinBox
 
 		StandardBasisStream (Field &F, size_t N) :
 			_field (F), _n (N), _j (0)
-		{ _field.init (_one, 1); }
+		{
+		}
 
 		Vector &get (Vector &v)
 		{
 			v.clear ();
 
 			if (_j < _n)
-				v.push_back (std::pair <size_t, typename Field::Element> (_j++, _one));
+				v.push_back (std::pair <size_t, typename Field::Element> (_j++, _field.one));
 
 			return v;
 		}
@@ -801,7 +787,6 @@ namespace LinBox
 		const Field              &_field;
 		size_t                    _n;
 		size_t                    _j;
-		typename Field::Element   _one;
 	};
 
 	//! Specialization of standard basis stream for sparse associative vectors
@@ -813,14 +798,15 @@ namespace LinBox
 
 		StandardBasisStream (Field &F, size_t N) :
 			_field (F), _n (N), _j (0)
-		{ _field.init (_one, 1); }
+		{
+		}
 
 		Vector &get (Vector &v)
 		{
 			v.clear ();
 
 			if (_j < _n)
-				v.insert (std::pair <size_t, typename Field::Element> (_j++, _one));
+				v.insert (std::pair <size_t, typename Field::Element> (_j++, _field.one));
 
 			return v;
 		}
@@ -839,7 +825,6 @@ namespace LinBox
 		const Field              &_field;
 		size_t                    _n;
 		size_t                    _j;
-		typename Field::Element   _one;
 	};
 
 	//! Specialization of standard basis stream for sparse parallel vectors
@@ -851,7 +836,7 @@ namespace LinBox
 
 		StandardBasisStream (Field &F, size_t N) :
 			_field (F), _n (N), _j (0)
-		{ _field.init (_one, 1); }
+		{  }
 
 		Vector &get (Vector &v)
 		{
@@ -860,7 +845,7 @@ namespace LinBox
 
 			if (_j < _n) {
 				v.first.push_back (_j++);
-				v.second.push_back (_one);
+				v.second.push_back (_field.one);
 			}
 
 			return v;
@@ -880,7 +865,6 @@ namespace LinBox
 		const Field              &_field;
 		size_t                    _n;
 		size_t                    _j;
-		typename Field::Element   _one;
 	};
 
 } // namespace LinBox
@@ -889,13 +873,10 @@ namespace LinBox
 
 #endif // __LINBOX_vector_stream_H
 
-
-
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
